/* gjacktransport - jack transport slider
 *  Copyright (C) 2006 Robin Gareus  <robin AT gareus.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <math.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "gjack.h"
#include "gjacktransport.h"

extern GtkWidget *window1;
extern gjtdata stg[MAX_MEM]; // settings
extern int mem;

int mode = 0;
gdouble z_pos = 50.0;
gdouble z_scale = 0.0;
gdouble fastskip = 0.0;

gdouble unit_table[UNIT_LAST] =  { 3600.0, 60.0, 1.0, 1.0/48000.0, 
	1.0/10.0, 1.0/14.0, 1.0/15.0, 1.0/23.976, 1.0/24.0, 1.0/24.976 ,1.0/25.0, 1.0/29.97, 1.0/30.0, 1.0/59.94 , 1.0/60 };

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

void initialize_gjt (void) {
	int i;
	for (i=0; i< MAX_MEM; i++) {
		stg[i].unit=UNIT_SEC;
		stg[i].start=0.0;
		stg[i].end=60.0;
	}
	stg[1].unit=UNIT_MIN;
	stg[1].end=3600.0;
	stg[2].unit=UNIT_HOUR;
	stg[2].end=2.5*3600.0;
}

void showhide (char *togglebtn, char *widget, gboolean display) {
	GtkWidget *t=lookup_widget(window1,togglebtn);
	GtkWidget *w=lookup_widget(window1,widget);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t),display);
	/* - is this [also] done by btn callback ? check */
	if (display) {
		gtk_widget_show (w);
	} else {
		gtk_widget_hide (w);
	}
}

gboolean ishidden (char *widget) {
        GtkWidget *w = lookup_widget(window1,widget);
	return (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
}

void dothejack (gboolean onoff) {
        GtkWidget *w = lookup_widget(window1,"checkbutton1");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),onoff);
}

void select_mem (int newmem) {
	if (newmem < 0 || newmem>= MAX_MEM) return;
	GtkWidget *w = lookup_widget(window1,"combobox5");
	gtk_combo_box_set_active (GTK_COMBO_BOX(w),newmem);
	// TODO ?? call on_combobox5_changed() ??
}

void update_startend (int newunit) {
	int oldunit=stg[mem].unit;
	gdouble val;
	gdouble min,max;
	stg[mem].unit=newunit; // NOTE: spinbtn callbacks use 'unit'

	GtkWidget *spbtn1 = lookup_widget(window1,"spinbutton1");
	GtkWidget *spbtn2 = lookup_widget(window1,"spinbutton2");

	gtk_spin_button_get_range (GTK_SPIN_BUTTON(spbtn2), &min, &max);
	gtk_spin_button_set_range (GTK_SPIN_BUTTON(spbtn2), 0, max);

	val = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spbtn2));
	val*=unit_table[oldunit]/unit_table[newunit];
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spbtn2),val);

	val = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spbtn1));
	val*=unit_table[oldunit]/unit_table[newunit];
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spbtn1),val);

}

void update_spinbtns (void) {
	GtkWidget *w;
	w = lookup_widget(window1,"spinbutton1");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w),stg[mem].start);
	w = lookup_widget(window1,"spinbutton2");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w),stg[mem].end);
}

void update_smpte (double sec) {
	gchar text[16];
	gfloat secdiv = 100.0; 
	if (stg[mem].unit > 3) secdiv=1.0/unit_table[stg[mem].unit];
	GtkWidget *label_smpte = lookup_widget(window1,"label_smpte");
	if (sec < 0) snprintf(text, 16 , "--:--:--.--");
	else snprintf(text, 16 , "%02i:%02i:%02i.%02i",
		(int)floor(sec/3600.0),
		(int)floor((sec-3600.0*floor(sec/3600.0))/60.0),
		(int)floor(sec-60.0*floor(sec/60.0)),
		(int)floor(secdiv*(sec-floor(sec))));
	gtk_label_set_text(GTK_LABEL(label_smpte),text);
}

void update_playpause_btn(void) {
	GtkWidget *w; 
	int val = jack_poll_transport_state();
	w = lookup_widget(window1,"button_pause");
	gtk_widget_set_sensitive(w,val&1);
	w = lookup_widget(window1,"button_play");
	gtk_widget_set_sensitive(w,val&2);
	w = lookup_widget(window1,"button_ff");
	gtk_widget_set_sensitive(w,val&4);
	w = lookup_widget(window1,"button_fr");
	gtk_widget_set_sensitive(w,val&4);
	w = lookup_widget(window1,"button_rewind");
	gtk_widget_set_sensitive(w,val&4);
	w = lookup_widget(window1,"button_end");
	gtk_widget_set_sensitive(w,val&4);
	w = lookup_widget(window1,"button_pos");
	gtk_widget_set_sensitive(w,val&4);
	w = lookup_widget(window1,"button6");
	gtk_widget_set_sensitive(w,val&4);
}

/* main-timeout function that moves the
   slider along with jack-transport */
gboolean gpolljack (gpointer data) {

	GtkWidget *scale = lookup_widget(window1,"hscale1");
	double val=500;
	double jt=jack_poll_time();
	update_playpause_btn();

	if (jt < 0 ) { return (TRUE);}
	update_smpte(jt);

	val = (jt-stg[mem].start)/(stg[mem].end-stg[mem].start)*1000.0;

	if (mode != 0 ) { return (TRUE);}
	gtk_range_set_value (GTK_RANGE(scale),val);
	return(TRUE);
}

gboolean gfastskip (gpointer data) {
	if (fastskip == 0.0) return (FALSE);

	GtkWidget *scale = lookup_widget(window1,"hscale1");
	mode=2;
	double val = gtk_range_get_value (GTK_RANGE(scale));
	gtk_range_set_value (GTK_RANGE(scale),val+fastskip);

#if 1
	if (fastskip < 100.0) 
		//fastskip+=(fastskip/5.0);
		fastskip*=1.3;
#endif
	return(TRUE);
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

void
on_hscale1_value_changed               (GtkRange        *range,
                                        gpointer         user_data)
{
	gdouble val = gtk_range_get_value (range);
	gdouble sec = val*(stg[mem].end-stg[mem].start)/1000.0 + stg[mem].start;
	if (mode == 0 ) { return;}
//	printf("jack reposition : val %.3f - %.3f sec \n",val,sec);
	jack_reposition(sec);
	if (mode == 2 ) {mode=0;}
}


void
on_checkbutton1_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        GtkWidget *widget = lookup_widget(window1,"image15");
	if ( gtk_toggle_button_get_active(togglebutton)) {
		open_jack();
	} else {
		close_jack();
	}
	if (jack_connected()) {
		gtk_image_set_from_icon_name(GTK_IMAGE(widget),"gtk-connect",GTK_ICON_SIZE_BUTTON);
		gtk_toggle_button_set_active(togglebutton,TRUE);
	} else  {
		gtk_image_set_from_icon_name(GTK_IMAGE(widget),"gtk-disconnect",GTK_ICON_SIZE_BUTTON);
		gtk_toggle_button_set_active(togglebutton,FALSE);
	}

}


void
on_spinbutton1_value_changed           (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
	//GtkWidget *scale = lookup_widget(window1,"hscale1");
	GtkWidget *spbtn1 = lookup_widget(window1,"spinbutton1");
	GtkWidget *spbtn2 = lookup_widget(window1,"spinbutton2");
	gdouble val = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spbtn1));
	gdouble min,max;
	gtk_spin_button_get_range (GTK_SPIN_BUTTON(spbtn2), &min, &max);
	gtk_spin_button_set_range (GTK_SPIN_BUTTON(spbtn2), val+(1.0/unit_table[stg[mem].unit]), max);

	stg[mem].start=val * unit_table[stg[mem].unit];
}


void
on_spinbutton2_value_changed           (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
	//GtkWidget *scale = lookup_widget(window1,"hscale1");
	GtkWidget *spbtn2 = lookup_widget(window1,"spinbutton2");
	gdouble val = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spbtn2));
	stg[mem].end=val * unit_table[stg[mem].unit];

//	on_hscale1_value_changed(GTK_RANGE(scale),NULL);
}


void
on_hscale1_move_slider                 (GtkRange        *range,
                                        GtkScrollType    scroll,
                                        gpointer         user_data)
{
	mode=2;
//	on_hscale1_value_changed(range,NULL);
}


gboolean
on_hscale1_button_press_event          (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	mode=1;
	return FALSE;
}


gboolean
on_hscale1_button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	mode=0;
	return FALSE;
}




gboolean
on_hscale1_change_value                (GtkRange        *range,
                                        GtkScrollType    scroll,
                                        gdouble          value,
                                        gpointer         user_data)
{
  return FALSE;
}


void
on_button_rewind_released              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *scale = lookup_widget(window1,"hscale1");
	mode=2;
	gtk_range_set_value (GTK_RANGE(scale),0.0);
}


void
on_button_play_released                (GtkButton       *button,
                                        gpointer         user_data)
{
	jack_play();
}


void
on_button_pause_released               (GtkButton       *button,
                                        gpointer         user_data)
{
	jack_stop();
}


void
on_button_ff_pressed                   (GtkButton       *button,
                                        gpointer         user_data)
{
	fastskip=4.0;
	gfastskip(NULL);
	g_timeout_add(120,gfastskip,NULL);
}


void
on_button_ff_released                  (GtkButton       *button,
                                        gpointer         user_data)
{
	fastskip=0.0;
#if 0
	GtkWidget *scale = lookup_widget(window1,"hscale1");
	mode=2;
	double val = gtk_range_get_value (GTK_RANGE(scale));
	gtk_range_set_value (GTK_RANGE(scale),val+40.0);
#endif
}


void
on_button_end_released                 (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *scale = lookup_widget(window1,"hscale1");
	mode=2;
	gtk_range_set_value (GTK_RANGE(scale),1000.0);
}


void
on_button_fr_pressed                   (GtkButton       *button,
                                        gpointer         user_data)
{
	fastskip=-4.0;
	gfastskip(NULL);
	g_timeout_add(120,gfastskip,NULL);
}


void
on_button_fr_released                  (GtkButton       *button,
                                        gpointer         user_data)
{
	fastskip=0.0;
#if 0
	GtkWidget *scale = lookup_widget(window1,"hscale1");
	mode=2;
	double val = gtk_range_get_value (GTK_RANGE(scale));
	gtk_range_set_value (GTK_RANGE(scale),val-40.0);
#endif
}


void
on_button6_released                    (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *spbtn1 = lookup_widget(window1,"spinbutton1");
	GtkWidget *spbtn2 = lookup_widget(window1,"spinbutton2");

	double val = jack_poll_time();
	if (z_scale == 0.0) {
		stg[mem].start=0.0;
		stg[mem].end   = val * 100.0 / z_pos;
		if (stg[mem].end==stg[mem].start) stg[mem].end+=60.0;
	} else {
		stg[mem].start = val - (z_scale / 100.0 * z_pos);
		stg[mem].end   = val + (z_scale / 100.0 * (100.0-z_pos));
	}

	if (stg[mem].start < 0.0 ) stg[mem].start=0.0;

	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spbtn2),stg[mem].end/unit_table[stg[mem].unit]);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spbtn1),stg[mem].start/unit_table[stg[mem].unit]);
}

void
on_combobox1_changed                   (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	guint sel = gtk_combo_box_get_active (combobox);
	if (sel+3 >= UNIT_LAST ) return;
	update_startend(4+ sel);
}


void
on_combobox2_changed                   (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	GtkWidget *w=lookup_widget(window1,"combobox1");
	guint sel = gtk_combo_box_get_active (combobox);

	if (sel > 4 ) return;
	if (sel < 0 ) return;
	gtk_widget_set_sensitive(w,(sel==3));

	if (sel < 3 ) { 
		update_startend(sel);
		return;
	}
	if (sel == 4 ) { 
		update_startend(UNIT_AUDIOFRAMES);
		return;
	} else { 
		sel = gtk_combo_box_get_active (GTK_COMBO_BOX(w));
		if (sel+3 >= UNIT_LAST ) return;
		update_startend(4+ sel);
	}
}


void
on_button_pos_released                 (GtkButton       *button,
                                        gpointer         user_data)
{
	jack_reposition(0);
}


void
on_combobox3_changed                   (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	 switch(gtk_combo_box_get_active (combobox)) {
	 	case 1: z_scale=1.0; break;
	 	case 2: z_scale=5.0; break;
	 	case 3: z_scale=10.0; break;
	 	case 4: z_scale=60.0; break;
	 	case 5: z_scale=300.0; break;
	 	case 6: z_scale=900.0; break;
	 	default: z_scale=0.0; 
	 }
}


void
on_combobox4_changed                   (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	 switch(gtk_combo_box_get_active (combobox)) {
	 	case 1: z_pos=50.0; break;
	 	case 2: z_pos=90.0; break;
	 	case 3: z_pos=100.0; break;
	 	default: z_pos=10.0;
	 }
}


void
on_checkbutton4_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *w=lookup_widget(window1,"handlebox1");
	if ( gtk_toggle_button_get_active(togglebutton)) 
		gtk_widget_show (w);
	else  
		gtk_widget_hide (w);
}


void
on_checkbutton2_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *w=lookup_widget(window1,"handlebox2");
	if ( gtk_toggle_button_get_active(togglebutton)) 
		gtk_widget_show (w);
	else  
		gtk_widget_hide (w);
}


void
on_checkbutton5_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *w=lookup_widget(window1,"statusbar1");
	if ( gtk_toggle_button_get_active(togglebutton)) 
		gtk_widget_show (w);
	else  
		gtk_widget_hide (w);
}


void
on_checkbutton3_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *w=lookup_widget(window1,"handlebox3");
	if ( gtk_toggle_button_get_active(togglebutton)) 
		gtk_widget_show (w);
	else  
		gtk_widget_hide (w);
}


void
on_checkbutton6_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *w=lookup_widget(window1,"table1");
	if ( gtk_toggle_button_get_active(togglebutton)) {
		gtk_widget_show (w);
		status_printf("");
	} else {
		gtk_widget_hide (w);
		// TODO: parse/print unit.
		status_printf("minimized: %.2f .. %.2f ",stg[mem].start,stg[mem].end);
	}
}


void
on_combobox5_changed                   (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	int oldmem = mem;
	int t=gtk_combo_box_get_active (combobox);
	if (t < 0 || t>= MAX_MEM) return;
	mem=t;
	if (oldmem == mem) return;

	// update start + end time
	GtkWidget *spbtn1 = lookup_widget(window1,"spinbutton1");
	GtkWidget *spbtn2 = lookup_widget(window1,"spinbutton2");
	GtkWidget *w;
	gdouble min,max;

	gtk_spin_button_get_range (GTK_SPIN_BUTTON(spbtn2), &min, &max);
	gtk_spin_button_set_range (GTK_SPIN_BUTTON(spbtn2), 0, max);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spbtn2),stg[mem].end/unit_table[stg[mem].unit]);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spbtn1),stg[mem].start/unit_table[stg[mem].unit]);
	
	// select fps unit
	w = lookup_widget(window1,"combobox1");
	if (stg[mem].unit>3)
		gtk_combo_box_set_active (GTK_COMBO_BOX(w),(stg[mem].unit>3)?stg[mem].unit-4:6);
	gtk_widget_set_sensitive(w,(stg[mem].unit>3));

	// select units
	w = lookup_widget(window1,"combobox2");
	if (stg[mem].unit==3) gtk_combo_box_set_active (GTK_COMBO_BOX(w),4);  // Audioframes
	else gtk_combo_box_set_active (GTK_COMBO_BOX(w),(stg[mem].unit<3)?stg[mem].unit:3); 

	// (also remember fit params ??)

}


void
on_checkbutton7_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *w=lookup_widget(window1,"handlebox4");
	if ( gtk_toggle_button_get_active(togglebutton)) 
		gtk_widget_show (w);
	else  
		gtk_widget_hide (w);
}

